/* $Id: cpl_memory.h,v 1.19 2010/03/11 16:54:17 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2005 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2010/03/11 16:54:17 $
 * $Revision: 1.19 $
 * $Name: cpl-5_2_0 $
 */

#ifndef CPL_MEMORY_H
#define CPL_MEMORY_H

#include <sys/types.h>
#include <stdarg.h>

#include <cpl_macros.h>

/*----------------------------------------------------------------------------
                                    Defines
 ----------------------------------------------------------------------------*/

#undef CPL_HAVE_GNUC_ALLOC_SIZE

#if defined __GNUC__
#if __GNUC__ > 4 || __GNUC__ == 4 && defined __GNUC_MINOR__ && __GNUC_MINOR__ > 2
#define CPL_HAVE_GNUC_ALLOC_SIZE
#endif
#endif

#undef CPL_HAVE_GNUC_UNUSED_RESULT

#if defined __GNUC__ &&  __GNUC__ > 3
#define CPL_HAVE_GNUC_UNUSED_RESULT
#endif


/*----------------------------------------------------------------------------
                            Function prototypes
 ----------------------------------------------------------------------------*/

CPL_BEGIN_DECLS

void *cpl_malloc(size_t)
#ifdef CPL_HAVE_GNUC_ALLOC_SIZE
    __attribute__((warn_unused_result, malloc, alloc_size(1)))
#else
    __attribute__((malloc))
#endif
;
void *cpl_calloc(size_t, size_t)
#ifdef CPL_HAVE_GNUC_ALLOC_SIZE
    __attribute__((warn_unused_result, malloc, alloc_size(1,2)))
#else
    __attribute__((malloc))
#endif
;
void *cpl_realloc(void *, size_t)
#ifdef CPL_HAVE_GNUC_ALLOC_SIZE
    __attribute__((warn_unused_result, alloc_size(2)))
#endif
;

void cpl_free(void *);

char *cpl_strdup(const char *)
#ifdef CPL_HAVE_GNUC_UNUSED_RESULT
    __attribute__((warn_unused_result, malloc))
#else
    __attribute__((malloc))
#endif
;
char *cpl_vsprintf(const char *, va_list)
#ifdef CPL_HAVE_GNUC_UNUSED_RESULT
    __attribute__((warn_unused_result, malloc, format (printf, 1, 0)))
#elif defined __GNUC__
    __attribute__((malloc, format (printf, 1, 0)))
#endif
     ;

char * cpl_sprintf(const char *, ...)
#ifdef CPL_HAVE_GNUC_UNUSED_RESULT
     __attribute__((warn_unused_result, malloc, format (printf, 1, 2)))
#elif defined __GNUC__
    __attribute__((malloc, format (printf, 1, 2)))
#endif
     ;

int cpl_memory_is_empty(void)
#ifdef __GNUC__
     __attribute__((pure))
#endif
     ;
void cpl_memory_dump(void);

CPL_END_DECLS

#endif /* CPL_MEMORY_H */
